@@ -329,24 +329,26 @@ static GHashTable *gst_rtp_bin_build_pt_map (const gchar *caps_string)
 {
   GHashTable *pt_map = NULL;
   gint i;
+  gint pt;
   GstCaps *caps = NULL;
+  GstCaps *new_caps = NULL;
+  GstStructure *structure = NULL;
+  GstStaticCaps static_caps = GST_STATIC_CAPS (caps_string);
+
+  caps = gst_static_caps_get (&static_caps);
 
   pt_map = g_hash_table_new_full (g_direct_hash,
       g_direct_equal,
       NULL,
       (GDestroyNotify) gst_caps_unref);
-
-  GstStaticCaps static_caps = GST_STATIC_CAPS (caps_string);
-  caps = gst_static_caps_get (&static_caps);
-
   for (i = 0; i < gst_caps_get_size (caps); i++)
   {
-    GstStructure *structure = gst_caps_get_structure (caps, i);
+    structure = gst_caps_get_structure (caps, i);
     GST_DEBUG ("Adding struct %p %s", structure,
         gst_structure_to_string (structure));
-    GstCaps *new_caps = gst_caps_new_full (gst_structure_copy (structure), NULL);
+
+    new_caps = gst_caps_new_full (gst_structure_copy (structure), NULL);
     GST_DEBUG ("Adding caps %p %s", new_caps, gst_caps_to_string (new_caps));
-    gint pt;
     if (gst_structure_get_int (structure, "payload", &pt))
     {
       g_hash_table_insert(pt_map, GINT_TO_POINTER(pt), (gpointer)
@@ -862,6 +864,7 @@ gst_rtp_bin_setup_rtpsend_pads(GstRTPBin * rtp_bin)
     internal_pad = gst_element_get_pad (rtp_bin->rtpsend, "datasink");
     newpad = gst_ghost_pad_new ("sink", internal_pad);
 
+    /* This could break with gstreamer 0.10.9 */
     gst_pad_set_active (newpad, TRUE);
 
     gst_object_unref (internal_pad);
@@ -873,15 +876,16 @@ gst_rtp_bin_setup_rtpsend_pads(GstRTPBin * rtp_bin)
 
     if (rtp_bin->bypass_udp)
     {
-        GST_DEBUG ("Using bypass_udp to create ghostpads for rtpsend");
         /* Let's just ghost pad rtpsrc on rtpbin */
         GstPad *bypass_pad = NULL;
         GstPad *tmppad = NULL;
 
+        GST_DEBUG ("Using bypass_udp to create ghostpads for rtpsend");
         tmppad = gst_element_get_static_pad (rtp_bin->rtpsend, "rtpsrc");
         bypass_pad = gst_ghost_pad_new ("rtpsrc", tmppad);
         gst_object_unref (tmppad);
 
+        /* This could break with gstreamer 0.10.9 */
         gst_pad_set_active (bypass_pad, TRUE);
 
         if (!gst_element_add_pad (GST_ELEMENT (rtp_bin), bypass_pad)) {
@@ -903,6 +907,7 @@ gst_rtp_bin_setup_rtprecv_pads(GstRTPBin * rtp_bin)
     newpad = gst_ghost_pad_new ("src",tmppad);
     gst_object_unref (tmppad);
 
+    /* This could break with gstreamer 0.10.9 */
     gst_pad_set_active (newpad, TRUE);
 
     if (!gst_element_add_pad (GST_ELEMENT (rtp_bin), newpad)) {
@@ -912,15 +917,16 @@ gst_rtp_bin_setup_rtprecv_pads(GstRTPBin * rtp_bin)
 
     if (rtp_bin->bypass_udp)
     {
-        GST_DEBUG ("Using bypass_udp to create ghostpads on rtprecv");
         /* Let's just ghost pad rtpsink on rtpbin */
         GstPad *newpad = NULL;
         GstPad *tmppad = NULL;
 
+        GST_DEBUG ("Using bypass_udp to create ghostpads on rtprecv");
         tmppad = gst_element_get_static_pad (rtp_bin->rtprecv, "rtpsink");
         newpad = gst_ghost_pad_new ("rtpsink",tmppad);
         gst_object_unref (tmppad);
 
+        /* This could break with gstreamer 0.10.9 */
         gst_pad_set_active (newpad, TRUE);
 
         if (!gst_element_add_pad (GST_ELEMENT (rtp_bin), newpad)) {
@@ -993,9 +999,11 @@ gst_rtp_bin_set_destinations (GstRTPBin *rtp_bin, const gchar *destination)
 
     for (i = 0; i < g_strv_length (ips); i++)
     {
-        GST_DEBUG("Found one address %s", ips[i]);
         gchar **split_ip;
         split_ip = g_strsplit (ips[i], ":", 2);
+
+        GST_DEBUG("Found one address %s", ips[i]);
+
         if (g_strv_length (split_ip) != 2)
         {
             GST_DEBUG("Badly formated address %s, use ip:port format", ips[i]);
@@ -1112,7 +1120,7 @@ gst_rtp_bin_set_property (GObject * object, guint prop_id,
         case ARG_QUEUE_DELAY:
             rtp_bin->queue_delay = g_value_get_uint (value);
             if (rtp_bin->jbuf) 
-              g_object_set (rtp_bin->jbuf, "queue_delay", rtp_bin->queue_delay,
+              g_object_set (rtp_bin->jbuf, "latency", rtp_bin->queue_delay,
                   NULL);
             break;
         case ARG_CLOSE_FD:
