@@ -136,6 +136,9 @@ static GstPad              *find_pad_for_pt            (GstRtpDemux   *rtpdemux,
 
 static GstFlowReturn       gst_rtp_demux_pad_unlinked  (GstRtpDemux   *rtpdemux,
                                                         GstPad        *pad);
+
+static void gst_rtp_demux_release_pad (GstElement *element, GstPad *pad);
+
 static guint gst_rtp_demux_signals[LAST_SIGNAL] = { 0 };
 
 static GstElementDetails gst_rtp_demux_details = {
@@ -196,6 +199,9 @@ gst_rtp_demux_class_init (GstRtpDemuxClass * klass)
 
   gstelement_klass->change_state =
       GST_DEBUG_FUNCPTR (gst_rtp_demux_change_state);
+
+  gstelement_klass->release_pad =
+      GST_DEBUG_FUNCPTR (gst_rtp_demux_release_pad);
 }
 
 static void
@@ -231,6 +237,12 @@ compare_srcpad (gconstpointer a, gconstpointer b)
   return first->pad - bpad;
 }
 
+static void
+gst_rtp_demux_release_pad (GstElement *element, GstPad *pad) {
+  GST_DEBUG_OBJECT (element, "Releasing pad %p of element %p", pad, element);
+  gst_rtp_demux_pad_unlinked (GST_RTP_DEMUX (element), pad);
+}
+
 static GstFlowReturn
 gst_rtp_demux_pad_unlinked (GstRtpDemux *rtpdemux, GstPad *pad)
 {
@@ -317,6 +329,7 @@ again:
 
     /* XXX: set _link () function */
     gst_pad_set_getcaps_function (srcpad, gst_rtp_demux_getcaps);
+    /* This could break with gstreamer 0.10.9 */
     gst_pad_set_active (srcpad, TRUE);
     gst_element_add_pad (element, srcpad);
 
