@@ -1,9 +1,9 @@
-/* 
+/*
  * Async Jitter Queue based on g_async_queue
  * This code is GST RTP smart and deals with timestamps
  *
  * Farsight Voice+Video library
- *  Copyright 2007 Collabora Ltd, 
+ *  Copyright 2007 Collabora Ltd,
  *  Copyright 2007 Nokia Corporation
  *   @author: Philippe Khalaf <philippe.khalaf@collabora.co.uk>.
  *
@@ -65,13 +65,14 @@ struct _AsyncJitterQueue
   gboolean pop_blocking;
   guint pops_remaining;
   guint32 tail_buffer_duration;
+  guint32 underruns;
 };
 
 /**
  * async_jitter_queue_new:
- * 
+ *
  * Creates a new asynchronous queue with the initial reference count of 1.
- * 
+ *
  * Return value: the new #AsyncJitterQueue.
  **/
 AsyncJitterQueue *
@@ -91,6 +92,7 @@ async_jitter_queue_new (void)
   retval->pop_blocking = TRUE;
   retval->pops_remaining = 0;
   retval->tail_buffer_duration = 0;
+  retval->underruns = 0;
   return retval;
 }
 
@@ -133,7 +135,7 @@ async_jitter_queue_ref (AsyncJitterQueue * queue)
 /**
  * async_jitter_queue_ref_unlocked:
  * @queue: a #AsyncJitterQueue.
- * 
+ *
  * Increases the reference count of the asynchronous @queue by 1.
  **/
 void
@@ -149,7 +151,7 @@ async_jitter_queue_ref_unlocked (AsyncJitterQueue * queue)
  * async_jitter_queue_set_low_threshold:
  * @queue: a #AsyncJitterQueue.
  * @threshold: the lower threshold (fraction of max size)
- * 
+ *
  * Sets the low threshold on the queue. This threshold indicates the minimum
  * number of items allowed in the queue before we refill it up to the set
  * maximum threshold.
@@ -168,7 +170,7 @@ async_jitter_queue_set_low_threshold (AsyncJitterQueue * queue,
  * async_jitter_queue_set_max_threshold:
  * @queue: a #AsyncJitterQueue.
  * @threshold: the higher threshold (fraction of max size)
- * 
+ *
  * Sets the high threshold on the queue. This threshold indicates the amount of
  * items to fill in the queue before releasing any blocking pop calls. This
  * blocking mecanism is only triggered when we reach the low threshold and must
@@ -184,6 +186,33 @@ async_jitter_queue_set_high_threshold (AsyncJitterQueue * queue,
   queue->high_threshold = threshold;
 }
 
+guint32
+async_jitter_queue_get_underruns (AsyncJitterQueue *queue)
+{
+  g_return_val_if_fail (queue, 0);
+  g_return_val_if_fail (g_atomic_int_get (&queue->ref_count) > 0, 0);
+
+  return queue->underruns;
+}
+
+gfloat
+async_jitter_queue_get_high_threshold (AsyncJitterQueue *queue)
+{
+  g_return_val_if_fail (queue, 0);
+  g_return_val_if_fail (g_atomic_int_get (&queue->ref_count) > 0, 0);
+
+  return queue->high_threshold;
+}
+
+gfloat
+async_jitter_queue_get_low_threshold (AsyncJitterQueue *queue)
+{
+  g_return_val_if_fail (queue, 0);
+  g_return_val_if_fail (g_atomic_int_get (&queue->ref_count) > 0, 0);
+
+  return queue->low_threshold;
+}
+
 /* set the maximum queue length in RTP timestamp units */
 void
 async_jitter_queue_set_max_queue_length (AsyncJitterQueue * queue,
@@ -230,7 +259,11 @@ async_jitter_queue_length_ts_units_unlocked (AsyncJitterQueue * queue)
 
   g_return_val_if_fail (queue, 0);
 
-  if (queue->queue->length < 2) {
+  /* FIXME we should be able to return the tail_buffer_duration if we have only
+   * one packet left as long as we can detect discontinuities in audio using the
+   * RTP marker bit */
+  if (queue->queue->length < 2)
+  {
     return 0;
   }
 
@@ -254,7 +287,7 @@ async_jitter_queue_length_ts_units_unlocked (AsyncJitterQueue * queue)
 /**
  * async_jitter_queue_unref_and_unlock:
  * @queue: a #AsyncJitterQueue.
- * 
+ *
  * Decreases the reference count of the asynchronous @queue by 1 and
  * releases the lock. This function must be called while holding the
  * @queue's lock. If the reference count went to 0, the @queue will be
@@ -273,7 +306,7 @@ async_jitter_queue_unref_and_unlock (AsyncJitterQueue * queue)
 /**
  * async_jitter_queue_unref:
  * @queue: a #AsyncJitterQueue.
- * 
+ *
  * Decreases the reference count of the asynchronous @queue by 1. If
  * the reference count went to 0, the @queue will be destroyed and the
  * memory allocated will be freed. So you are not allowed to use the
@@ -299,7 +332,7 @@ async_jitter_queue_unref (AsyncJitterQueue * queue)
 /**
  * async_jitter_queue_lock:
  * @queue: a #AsyncJitterQueue.
- * 
+ *
  * Acquires the @queue's lock. After that you can only call the
  * <function>async_jitter_queue_*_unlocked()</function> function variants on that
  * @queue. Otherwise it will deadlock.
@@ -316,7 +349,7 @@ async_jitter_queue_lock (AsyncJitterQueue * queue)
 /**
  * async_jitter_queue_unlock:
  * @queue: a #AsyncJitterQueue.
- * 
+ *
  * Releases the queue's lock.
  **/
 void
@@ -351,7 +384,7 @@ async_jitter_queue_push (AsyncJitterQueue * queue, gpointer data)
  * async_jitter_queue_push_unlocked:
  * @queue: a #AsyncJitterQueue.
  * @data: @data to push into the @queue.
- * 
+ *
  * Pushes the @data into the @queue. @data must not be %NULL. This
  * function must be called while holding the @queue's lock.
  **/
@@ -377,17 +410,17 @@ async_jitter_queue_push_unlocked (AsyncJitterQueue * queue, gpointer data)
  *     should be higher in the @queue or a positive value if the first
  *     element should be lower in the @queue than the second element.
  * @user_data: user data passed to @func.
- * 
+ *
  * Inserts @data into @queue using @func to determine the new
- * position. 
- * 
+ * position.
+ *
  * This function requires that the @queue is sorted before pushing on
  * new elements.
- * 
+ *
  * This function will lock @queue before it sorts the queue and unlock
  * it when it is finished.
- * 
- * For an example of @func see async_jitter_queue_sort(). 
+ *
+ * For an example of @func see async_jitter_queue_sort().
  *
  * Since: 2.10
  **/
@@ -395,9 +428,10 @@ gboolean
 async_jitter_queue_push_sorted (AsyncJitterQueue * queue,
     gpointer data, GCompareDataFunc func, gpointer user_data)
 {
-  g_return_val_if_fail (queue != NULL, FALSE);
   gboolean ret;
 
+  g_return_val_if_fail (queue != NULL, FALSE);
+
   g_mutex_lock (queue->mutex);
   ret = async_jitter_queue_push_sorted_unlocked (queue, data, func, user_data);
   g_mutex_unlock (queue->mutex);
@@ -415,19 +449,19 @@ async_jitter_queue_push_sorted (AsyncJitterQueue * queue,
  *     should be higher in the @queue or a positive value if the first
  *     element should be lower in the @queue than the second element.
  * @user_data: user data passed to @func.
- * 
+ *
  * Inserts @data into @queue using @func to determine the new
  * position.
- * 
+ *
  * This function requires that the @queue is sorted before pushing on
  * new elements.
  *
  * If @GCompareDataFunc returns 0, this function does not insert @data and
  * return FALSE.
- * 
+ *
  * This function is called while holding the @queue's lock.
- * 
- * For an example of @func see async_jitter_queue_sort(). 
+ *
+ * For an example of @func see async_jitter_queue_sort().
  *
  * Since: 2.10
  **/
@@ -477,25 +511,46 @@ async_jitter_queue_pop_intern_unlocked (AsyncJitterQueue * queue)
   if (queue->pop_flushing)
     return NULL;
 
-  while (queue->pop_blocking) {
-    queue->waiting_threads++;
-    g_cond_wait (queue->cond, queue->mutex);
-    queue->waiting_threads--;
-    if (queue->pop_flushing)
-      return NULL;
-  }
-
-  if (async_jitter_queue_length_ts_units_unlocked (queue) <=
-      queue->low_threshold * queue->max_queue_length
-      && queue->pops_remaining == 0) {
-    if (!queue->buffering) {
-      queue->buffering = TRUE;
-      queue->pops_remaining = queue->queue->length;
-    } else {
-      while (!g_queue_peek_tail (queue->queue) || queue->pop_blocking) {
+  /* if pops are remaining, do it now without blocking */
+  if (queue->pops_remaining == 0)
+  {
+    /* let's check if we are under our low threshold */
+    if (async_jitter_queue_length_ts_units_unlocked (queue) <=
+        queue->low_threshold * queue->max_queue_length)
+    {
+      if (!queue->buffering)
+      {
+        //g_debug ("Turned on buffering");
+        queue->buffering = TRUE;
+        /* this will make sure we release all remaining buffers from the queue
+         * before starting to buffer */
+        queue->pops_remaining = queue->queue->length;
+      }
+      else
+      {
+        /* Let's wait until the high threshold is reached */
+        while (!g_queue_peek_tail (queue->queue) || queue->pop_blocking) {
+          g_debug ("Sink needs buffer, must wait for buffering to stop");
+          queue->underruns++;
+          queue->waiting_threads++;
+          g_cond_wait(queue->cond, queue->mutex);
+          queue->waiting_threads--;
+          //g_debug ("done waiting");
+          if (queue->pop_flushing)
+            return NULL;
+        }
+      }
+    }
+    /* Let's see if there is a buffer, if not, wait for signal */
+    else
+    {
+      while (queue->pop_blocking && !g_queue_peek_tail (queue->queue)) {
+        g_debug ("No packets available for sink! waiting");
         queue->waiting_threads++;
-        g_cond_wait (queue->cond, queue->mutex);
+        queue->underruns++;
+        g_cond_wait(queue->cond, queue->mutex);
         queue->waiting_threads--;
+        //g_debug ("done waiting");
         if (queue->pop_flushing)
           return NULL;
       }
@@ -506,15 +561,26 @@ async_jitter_queue_pop_intern_unlocked (AsyncJitterQueue * queue)
   if (queue->pops_remaining)
     queue->pops_remaining--;
 
+  /* Let's calculate the duration of the next buffer to be popped based on the
+   * buffer we just popped */
   tail_buffer = g_queue_peek_tail (queue->queue);
   if (tail_buffer) {
     if (!GST_IS_BUFFER (tail_buffer) || !GST_IS_BUFFER (retval)) {
       queue->tail_buffer_duration = 0;
     } else if (gst_rtp_buffer_get_seq (tail_buffer)
         - gst_rtp_buffer_get_seq (retval) == 1) {
-      queue->tail_buffer_duration =
-          calculate_ts_diff (gst_rtp_buffer_get_timestamp (tail_buffer),
+      queue->tail_buffer_duration = calculate_ts_diff (
+          gst_rtp_buffer_get_timestamp (tail_buffer),
           gst_rtp_buffer_get_timestamp (retval));
+      /* TODO */
+      /* double check on the commented code. check the RTP specs to make sure
+       * the market bit always means an audio discontinuity, if so, we can't
+       * calculate the tail buffer duration even if the seq-num are subsequent
+      if (gst_rtp_buffer_get_marker (tail_buffer))
+      {
+        queue->tail_buffer_duration = 0;
+      }
+      */
     } else {
       /* There is a sequence number gap -> we can't calculate the duration
        * let's just set it to 0 */
@@ -530,7 +596,7 @@ async_jitter_queue_pop_intern_unlocked (AsyncJitterQueue * queue)
 /**
  * async_jitter_queue_pop:
  * @queue: a #AsyncJitterQueue.
- * 
+ *
  * Pops data from the @queue. This function blocks until data become
  * available. If pop is disabled, tis function return NULL.
  *
@@ -554,7 +620,7 @@ async_jitter_queue_pop (AsyncJitterQueue * queue)
 /**
  * async_jitter_queue_pop_unlocked:
  * @queue: a #AsyncJitterQueue.
- * 
+ *
  * Pops data from the @queue. This function blocks until data become
  * available. This function must be called while holding the @queue's
  * lock.
@@ -573,7 +639,7 @@ async_jitter_queue_pop_unlocked (AsyncJitterQueue * queue)
 /**
  * async_jitter_queue_length:
  * @queue: a #AsyncJitterQueue.
- * 
+ *
  * Returns the length of the queue
  * Return value: the length of the @queue.
  **/
@@ -614,7 +680,7 @@ async_jitter_queue_length_unlocked (AsyncJitterQueue * queue)
  * @queue: a #AsyncJitterQueue.
  * @free_func: a function to call to free the elements
  * @user_data: user data passed to @free_func
- * 
+ *
  * This function is used to set/unset flushing. If flushing is set any
  * waiting/blocked pops will be unblocked. Any subsequent calls to pop will
  * return NULL. Flushing is set by default.
@@ -623,6 +689,8 @@ void
 async_jitter_queue_set_flushing_unlocked (AsyncJitterQueue * queue,
     GFunc free_func, gpointer user_data)
 {
+  gpointer elem;
+
   g_return_if_fail (queue);
   g_return_if_fail (g_atomic_int_get (&queue->ref_count) > 0);
 
@@ -631,7 +699,8 @@ async_jitter_queue_set_flushing_unlocked (AsyncJitterQueue * queue,
   if (queue->waiting_threads > 0)
     g_cond_broadcast (queue->cond);
   /* free data from queue */
-  g_queue_foreach (queue->queue, free_func, user_data);
+  while ((elem = g_queue_pop_head (queue->queue)))
+    free_func (elem, user_data);
 }
 
 /**
@@ -639,7 +708,7 @@ async_jitter_queue_set_flushing_unlocked (AsyncJitterQueue * queue,
  * @queue: a #AsyncJitterQueue.
  * @free_func: a function to call to free the elements
  * @user_data: user data passed to @free_func
- * 
+ *
  * This function is used to set/unset flushing. If flushing is set any
  * waiting/blocked pops will be unblocked. Any subsequent calls to pop will
  * return NULL. Flushing is set by default.
@@ -660,7 +729,7 @@ async_jitter_queue_unset_flushing_unlocked (AsyncJitterQueue * queue)
  * async_jitter_queue_set_blocking_unlocked:
  * @queue: a #AsyncJitterQueue.
  * @enabled: a boolean to enable/disable blocking
- * 
+ *
  * This function is used to enable/disable blocking. If blocking is enabled any
  * pops will be blocked until the queue is unblocked. The queue is blocked by
  * default.
