@@ -33,12 +33,18 @@
  * This element reorders and removes duplicate RTP packets as they are received
  * from a network source. It will also wait for missing packets up to a
  * configurable time limit using the ::latency property. Packets arriving too
- * late are considered as lost packets.
+ * late are considered to be lost packets.
  * </para>
  * <para>
  * This element acts as a live element and so adds ::latency to the pipeline.
  * </para>
  * <para>
+ * The element needs the clock-rate of the RTP payload in order to estimate the
+ * delay. This information is obtained either from the caps on the sink pad or,
+ * when no caps are present, from the ::request-pt-map signal. To clear the
+ * previous pt-map use the ::clear-pt-map signal.
+ * </para>
+ * <para>
  * This element also provides the application the ability to query run-time
  * statistics using the custom query, "jitterbuffer-statistics" on it's sinkpad.
  * On receiving the query, the jitterbuffer will fill the sturcture inside the
@@ -103,7 +109,7 @@
  * <title>Example pipelines</title>
  * <para>
  * <programlisting>
- * gst-launch rtspsrc location=rtsp://192.168.1.133:8554/mpeg1or2AudioVideoTest ! rtpjitterbuffer ! rtpmpvdepay ! mpeg2dec ! xvimagesink
+ * gst-launch rtspsrc location=rtsp://192.168.1.133:8554/mpeg1or2AudioVideoTest ! gstrtpjitterbuffer ! rtpmpvdepay ! mpeg2dec ! xvimagesink
  * </programlisting>
  * Connect to a streaming server and decode the MPEG video. The jitterbuffer is
  * inserted into the pipeline to smooth out network jitter and to reorder the
@@ -111,13 +117,14 @@
  * </para>
  * </refsect2>
  *
- * Last reviewed on 2007-03-27 (0.10.13)
+ * Last reviewed on 2007-05-28 (0.10.5)
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#include <stdlib.h>
 #include <string.h>
 #include <gst/rtp/gstrtpbuffer.h>
 
@@ -130,15 +137,15 @@ GST_DEBUG_CATEGORY (rtpjitterbuffer_debug);
 #define GST_CAT_DEFAULT (rtpjitterbuffer_debug)
 
 /* low and high threshold tell the queue when to start and stop buffering */
-#define LOW_THRESHOLD 0.2
-#define HIGH_THRESHOLD 0.8
+#define LOW_THRESHOLD 0.1
+#define HIGH_THRESHOLD 0.3
 
 static GstQueryType stats_query_type = GST_QUERY_NONE;
 
 /* elementfactory information */
 static const GstElementDetails gst_rtp_jitter_buffer_details =
 GST_ELEMENT_DETAILS ("RTP packet jitter-buffer",
-    "Filter/Network",
+    "Filter/Network/RTP",
     "A buffer that deals with network jitter and other transmission faults",
     "Philippe Kalaf <philippe.kalaf@collabora.co.uk>, "
     "Wim Taymans <wim@fluendo.com>");
@@ -146,12 +153,12 @@ GST_ELEMENT_DETAILS ("RTP packet jitter-buffer",
 /* RTPJitterBuffer signals and args */
 enum
 {
-  /* FILL ME */
   SIGNAL_REQUEST_PT_MAP,
+  SIGNAL_CLEAR_PT_MAP,
   LAST_SIGNAL
 };
 
-#define DEFAULT_LATENCY_MS      200
+#define DEFAULT_LATENCY_MS      500
 #define DEFAULT_DROP_ON_LATENCY FALSE
 
 #define GST_QUERY_JB_STATS      "jitterbuffer-statistics"
@@ -159,9 +166,11 @@ enum
 
 enum
 {
-  ARG_0,
-  ARG_LATENCY,
-  ARG_DROP_ON_LATENCY
+  PROP_0,
+  PROP_LATENCY,
+  PROP_LOW_THRESHOLD,
+  PROP_HIGH_THRESHOLD,
+  PROP_DROP_ON_LATENCY
 };
 
 struct _GstRTPJitterBufferPrivate
@@ -196,7 +205,6 @@ struct _GstRTPJitterBufferPrivate
   guint64 num_late;
   guint64 num_duplicates;
   guint64 times_overrun;
-  guint64 times_underrun;
 };
 
 #define GST_RTP_JITTER_BUFFER_GET_PRIVATE(o) \
@@ -258,6 +266,9 @@ static void gst_rtp_jitter_buffer_loop (GstRTPJitterBuffer * jitterbuffer);
 static gboolean gst_rtp_jitter_buffer_src_query (GstPad * pad, GstQuery * query);
 static gboolean gst_rtp_jitter_buffer_sink_query (GstPad * pad, GstQuery * query);
 
+static void
+gst_rtp_jitter_buffer_clear_pt_map (GstRTPJitterBuffer * jitterbuffer);
+
 static void
 gst_rtp_jitter_buffer_base_init (gpointer klass)
 {
@@ -290,17 +301,36 @@ gst_rtp_jitter_buffer_class_init (GstRTPJitterBufferClass * klass)
   gobject_class->set_property = gst_rtp_jitter_buffer_set_property;
   gobject_class->get_property = gst_rtp_jitter_buffer_get_property;
 
-  g_object_class_install_property (gobject_class, ARG_LATENCY,
+  /**
+   * GstRTPJitterBuffer::latency:
+   * 
+   * The maximum latency of the jitterbuffer. Packets will be kept in the buffer
+   * for at most this time.
+   */
+  g_object_class_install_property (gobject_class, PROP_LATENCY,
       g_param_spec_uint ("latency", "Buffer latency in ms",
           "Amount of ms to buffer", 0, G_MAXUINT, DEFAULT_LATENCY_MS,
           G_PARAM_READWRITE));
-
-  g_object_class_install_property (gobject_class, ARG_DROP_ON_LATENCY,
-      g_param_spec_boolean ("drop_on_latency",
+  /**
+   * GstRTPJitterBuffer::drop-on-latency:
+   * 
+   * Drop oldest buffers when the queue is completely filled. 
+   */
+  g_object_class_install_property (gobject_class, PROP_DROP_ON_LATENCY,
+      g_param_spec_boolean ("drop-on-latency",
           "Drop buffers when maximum latency is reached",
           "Tells the jitterbuffer to never exceed the given latency in size",
           DEFAULT_DROP_ON_LATENCY, G_PARAM_READWRITE));
 
+  g_object_class_install_property (gobject_class, PROP_HIGH_THRESHOLD,
+      g_param_spec_float ("high_threshold", "The high threshold",
+          "Tells the jitterbuffer when to stop buffering", 0, 1,
+          HIGH_THRESHOLD, G_PARAM_READWRITE));
+
+  g_object_class_install_property (gobject_class, PROP_LOW_THRESHOLD,
+      g_param_spec_float ("low_threshold", "The low threshold",
+          "Tells the jitterbuffer when to start buffering", 0, 1,
+          LOW_THRESHOLD, G_PARAM_READWRITE));
   /**
    * GstRTPJitterBuffer::request-pt-map:
    * @buffer: the object which received the signal
@@ -313,9 +343,23 @@ gst_rtp_jitter_buffer_class_init (GstRTPJitterBufferClass * klass)
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstRTPJitterBufferClass,
           request_pt_map), NULL, NULL, gstrtpjitterbuffer_marshal_BOXED__UINT,
       GST_TYPE_CAPS, 1, G_TYPE_UINT);
+  /**
+   * GstRTPJitterBuffer::clear-pt-map:
+   * @buffer: the object which received the signal
+   *
+   * Invalidate the clock-rate as obtained with the ::request-pt-map signal.
+   */
+  gst_rtp_jitter_buffer_signals[SIGNAL_CLEAR_PT_MAP] =
+      g_signal_new ("clear-pt-map", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstRTPJitterBufferClass,
+          clear_pt_map), NULL, NULL, g_cclosure_marshal_VOID__VOID,
+      G_TYPE_NONE, 0, G_TYPE_NONE);
+
 
   gstelement_class->change_state = gst_rtp_jitter_buffer_change_state;
 
+  klass->clear_pt_map = GST_DEBUG_FUNCPTR (gst_rtp_jitter_buffer_clear_pt_map);
+
   GST_DEBUG_CATEGORY_INIT
       (rtpjitterbuffer_debug, "rtpjitterbuffer", 0, "RTP Jitter Buffer");
 }
@@ -385,6 +429,17 @@ gst_rtp_jitter_buffer_dispose (GObject * object)
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
+static void
+gst_rtp_jitter_buffer_clear_pt_map (GstRTPJitterBuffer * jitterbuffer)
+{
+  GstRTPJitterBufferPrivate *priv;
+
+  priv = jitterbuffer->priv;
+
+  /* this will trigger a new pt-map request signal, FIXME, do something better. */
+  priv->clock_rate = -1;
+}
+
 static GstCaps *
 gst_rtp_jitter_buffer_getcaps (GstPad * pad)
 {
@@ -451,7 +506,8 @@ gst_jitter_buffer_sink_parse_caps (GstRTPJitterBuffer * jitterbuffer,
   value = gst_structure_get_value (caps_struct, "clock-base");
   if (value && G_VALUE_HOLDS_UINT (value)) {
     priv->clock_base = g_value_get_uint (value);
-    GST_DEBUG_OBJECT (jitterbuffer, "got clock-base %d", priv->clock_base);
+    GST_DEBUG_OBJECT (jitterbuffer, "got clock-base %" G_GINT64_FORMAT,
+        priv->clock_base);
   } else
     priv->clock_base = -1;
 
@@ -547,6 +603,7 @@ gst_rtp_jitter_buffer_flush_stop (GstRTPJitterBuffer * jitterbuffer)
   gst_segment_init (&priv->segment, GST_FORMAT_TIME);
   priv->last_popped_seqnum = -1;
   priv->next_seqnum = -1;
+  priv->clock_rate = -1;
   /* allow pops from the src pad task */
   async_jitter_queue_unset_flushing_unlocked (jitterbuffer->priv->queue);
   async_jitter_queue_unlock (priv->queue);
@@ -633,6 +690,8 @@ gst_rtp_jitter_buffer_change_state (GstElement * element,
       async_jitter_queue_set_blocking_unlocked (jitterbuffer->priv->queue,
           TRUE);
       async_jitter_queue_unlock (priv->queue);
+      if (ret != GST_STATE_CHANGE_FAILURE)
+        ret = GST_STATE_CHANGE_NO_PREROLL;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
       break;
@@ -694,6 +753,8 @@ gst_rtp_jitter_buffer_sink_event (GstPad * pad, GstEvent * event)
   jitterbuffer = GST_RTP_JITTER_BUFFER (gst_pad_get_parent (pad));
   priv = jitterbuffer->priv;
 
+  GST_DEBUG_OBJECT (jitterbuffer, "received %s", GST_EVENT_TYPE_NAME (event));
+
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_NEWSEGMENT:
     {
@@ -726,14 +787,17 @@ gst_rtp_jitter_buffer_sink_event (GstPad * pad, GstEvent * event)
     }
     case GST_EVENT_FLUSH_START:
       gst_rtp_jitter_buffer_flush_start (jitterbuffer);
+      ret = gst_pad_push_event (priv->srcpad, event);
       break;
     case GST_EVENT_FLUSH_STOP:
-      gst_rtp_jitter_buffer_flush_stop (jitterbuffer);
+      ret = gst_pad_push_event (priv->srcpad, event);
+      ret = gst_rtp_jitter_buffer_src_activate_push (priv->srcpad, TRUE);
       break;
     case GST_EVENT_EOS:
     {
       /* push EOS in queue. We always push it at the head */
       async_jitter_queue_lock (priv->queue);
+      GST_DEBUG_OBJECT (jitterbuffer, "queuing EOS");
       /* check for flushing, we need to discard the event and return FALSE when
        * we are flushing */
       ret = priv->srcresult == GST_FLOW_OK;
@@ -854,16 +918,25 @@ gst_rtp_jitter_buffer_chain (GstPad * pad, GstBuffer * buffer)
    * is set. */
   if (priv->drop_on_latency) {
     if (async_jitter_queue_length_ts_units_unlocked (priv->queue) >=
-      priv->latency_ms * priv->clock_rate / 1000) {
-      GstBuffer *old_buf;
+        priv->latency_ms * priv->clock_rate / 1000) {
+      guint32 latency_ts_units;
   
       priv->times_overrun++;
       GST_DEBUG_OBJECT (jitterbuffer, "Queue full, dropping old packet #%d",
           seqnum);
 
-      old_buf = async_jitter_queue_pop_unlocked (priv->queue);
-      if (old_buf)
-        gst_buffer_unref (old_buf);
+      latency_ts_units = priv->latency_ms * priv->clock_rate / 1000;
+      while (async_jitter_queue_length_ts_units_unlocked (priv->queue) >=
+          latency_ts_units / 2)
+      {
+        GstBuffer *old_buf;
+        GST_DEBUG_OBJECT (jitterbuffer, "dropping buffer down to 0.5 %u %u", latency_ts_units,
+            async_jitter_queue_length_ts_units_unlocked (priv->queue));
+
+        old_buf = async_jitter_queue_pop_unlocked (priv->queue);
+        if (old_buf)
+          gst_buffer_unref (old_buf);
+      }
     }
   }
 
@@ -949,18 +1022,18 @@ gst_rtp_jitter_buffer_loop (GstRTPJitterBuffer * jitterbuffer)
   GstBuffer *outbuf;
   GstFlowReturn result;
   guint16 seqnum;
+#if 0
   guint32 rtp_time;
   GstClockTime timestamp;
   gint64 running_time;
+#endif
 
   priv = jitterbuffer->priv;
 
   async_jitter_queue_lock (priv->queue);
+#if 0
 again:
-  /* FIXME: is there a better way to detect underruns? */
-  if (async_jitter_queue_length_unlocked (priv->queue) <= 0) {
-    priv->times_underrun++;
-  }
+#endif
 
   GST_DEBUG_OBJECT (jitterbuffer, "Popping item");
   /* pop a buffer, we will get NULL if the queue was shut down */
@@ -1007,6 +1080,7 @@ again:
    * determine if we have missing a packet. If we have a missing packet (which
    * must be before this packet) we can wait for it until the deadline for this
    * packet expires. */
+#if 0
   if (priv->next_seqnum == -1 || priv->next_seqnum != seqnum) {
     GstClockID id;
     GstClockTimeDiff jitter;
@@ -1029,8 +1103,8 @@ again:
      * of the currently popped packet */
     rtp_time = gst_rtp_buffer_get_timestamp (outbuf);
 
-    GST_DEBUG_OBJECT (jitterbuffer, "rtp_time %u, base %u", rtp_time,
-        priv->clock_base);
+    GST_DEBUG_OBJECT (jitterbuffer, "rtp_time %u, base %" G_GINT64_FORMAT,
+        rtp_time, priv->clock_base);
 
     /* if no clock_base was given, take first ts as base */
     if (priv->clock_base == -1)
@@ -1042,8 +1116,9 @@ again:
     /* bring timestamp to gst time */
     timestamp = gst_util_uint64_scale (GST_SECOND, rtp_time, priv->clock_rate);
 
-    GST_DEBUG_OBJECT (jitterbuffer, "timestamp %" GST_TIME_FORMAT,
-        GST_TIME_ARGS (timestamp));
+    GST_DEBUG_OBJECT (jitterbuffer,
+        "rtptime %u, clock-rate %u, timestamp %" GST_TIME_FORMAT, rtp_time,
+        priv->clock_rate, GST_TIME_ARGS (timestamp));
 
     /* bring to running time */
     running_time = gst_segment_to_running_time (&priv->segment, GST_FORMAT_TIME,
@@ -1106,6 +1181,7 @@ again:
     }
   }
 push_buffer:
+#endif
   /* check if we are pushing something unexpected */
   if (priv->next_seqnum != -1 && priv->next_seqnum != seqnum) {
     gint dropped;
@@ -1148,6 +1224,7 @@ no_elem:
     async_jitter_queue_unlock (priv->queue);
     return;
   }
+#if 0
 flushing:
   {
     GST_DEBUG_OBJECT (jitterbuffer, "we are flushing");
@@ -1155,10 +1232,13 @@ flushing:
     async_jitter_queue_unlock (priv->queue);
     return;
   }
+#endif
 pause:
   {
-    GST_DEBUG_OBJECT (jitterbuffer, "pausing task, reason %s",
-                    gst_flow_get_name (result));
+    const gchar *reason;
+    reason = gst_flow_get_name (result);
+
+    GST_DEBUG_OBJECT (jitterbuffer, "pausing task, reason %s", reason);
 
     async_jitter_queue_lock (priv->queue);
     /* store result */
@@ -1196,6 +1276,10 @@ gst_rtp_jitter_buffer_src_query (GstPad * pad, GstQuery * query)
         if ((res = gst_pad_query (peer, query))) {
           gst_query_parse_latency (query, &us_live, &min_latency, &max_latency);
 
+          GST_DEBUG_OBJECT (jitterbuffer, "Peer latency: min %"
+              GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
+              GST_TIME_ARGS (min_latency), GST_TIME_ARGS (max_latency));
+
           min_latency += priv->latency_ms * GST_MSECOND;
           max_latency += priv->latency_ms * GST_MSECOND;
 
@@ -1211,6 +1295,7 @@ gst_rtp_jitter_buffer_src_query (GstPad * pad, GstQuery * query)
     }
 #endif
     default:
+      res = gst_pad_query_default (pad, query);
       break;
   }
 
@@ -1255,16 +1340,17 @@ gst_rtp_jitter_buffer_sink_query (GstPad * pad, GstQuery * query)
 
   if (GST_QUERY_TYPE (query) == stats_query_type) {
     GstStructure *structure;
-    guint64 fill_level;
+    guint64 fill_level, underruns;
 
     fill_level = gst_rtp_jitter_buffer_get_fill_level (jitterbuffer);
+    underruns = async_jitter_queue_get_underruns (priv->queue);
     structure = gst_query_get_structure (query);
     gst_structure_set (structure,
                     "total-packets", G_TYPE_UINT64, priv->num_packets,
                     "late-packets", G_TYPE_UINT64, priv->num_late,
                     "duplicate-packets", G_TYPE_UINT64, priv->num_duplicates,
                     "times-overrun", G_TYPE_UINT64, priv->times_overrun,
-                    "times-underrun", G_TYPE_UINT64, priv->times_underrun,
+                    "times-underrun", G_TYPE_UINT64, underruns,
                     "fill-level", G_TYPE_UINT, fill_level,
                     NULL);
     res = TRUE;
@@ -1282,7 +1368,7 @@ gst_rtp_jitter_buffer_set_property (GObject * object,
   GstRTPJitterBuffer *jitterbuffer = GST_RTP_JITTER_BUFFER (object);
 
   switch (prop_id) {
-    case ARG_LATENCY:
+    case PROP_LATENCY:
     {
       guint new_latency, old_latency;
 
@@ -1306,7 +1392,27 @@ gst_rtp_jitter_buffer_set_property (GObject * object,
 #endif
       break;
     }
-    case ARG_DROP_ON_LATENCY:
+    case PROP_HIGH_THRESHOLD:
+    {
+      gfloat high_threshold = g_value_get_float (value);
+      if (high_threshold >= 0 && high_threshold <= 1)
+      {
+        async_jitter_queue_set_high_threshold (jitterbuffer->priv->queue,
+            high_threshold);
+      }
+      break;
+    }
+    case PROP_LOW_THRESHOLD:
+    {
+      gfloat low_threshold = g_value_get_float (value);
+      if (low_threshold >= 0 && low_threshold <= 1)
+      {
+        async_jitter_queue_set_low_threshold (jitterbuffer->priv->queue,
+            low_threshold);
+      }
+      break;
+    }
+    case PROP_DROP_ON_LATENCY:
     {
       jitterbuffer->priv->drop_on_latency = g_value_get_boolean (value);
       break;
@@ -1324,10 +1430,18 @@ gst_rtp_jitter_buffer_get_property (GObject * object,
   GstRTPJitterBuffer *jitterbuffer = GST_RTP_JITTER_BUFFER (object);
 
   switch (prop_id) {
-    case ARG_LATENCY:
+    case PROP_LATENCY:
       g_value_set_uint (value, jitterbuffer->priv->latency_ms);
       break;
-    case ARG_DROP_ON_LATENCY:
+    case PROP_HIGH_THRESHOLD:
+      g_value_set_float (value, async_jitter_queue_get_high_threshold
+          (jitterbuffer->priv->queue));
+      break;
+    case PROP_LOW_THRESHOLD:
+      g_value_set_float (value, async_jitter_queue_get_low_threshold
+          (jitterbuffer->priv->queue));
+      break;
+    case PROP_DROP_ON_LATENCY:
       g_value_set_boolean (value, jitterbuffer->priv->drop_on_latency);
       break;
     default:
