@@ -24,6 +24,14 @@
 #include "gstr263depayload.h"
 #include "gstmsgsmpayload.h"
 #include "gstmsgsmdepayload.h"
+#include "gstdvpayload.h"
+#include "gstdvdepayload.h"
+#include "gstrtpcnpayload.h"
+#include "gstrtpcndepayload.h"
+#ifdef ENABLE_G729
+#include "gstrtpg729pay.h"
+#include "gstrtpg729depay.h"
+#endif
 
 static gboolean
 plugin_init (GstPlugin * plugin)
@@ -34,6 +42,20 @@ plugin_init (GstPlugin * plugin)
     return FALSE;
   if (!gst_r263depayload_plugin_init (plugin))
     return FALSE;
+  if (!gst_dvpayload_plugin_init (plugin))
+    return FALSE;
+  if (!gst_dvdepayload_plugin_init (plugin))
+    return FALSE;
+  if (!gst_rtpcnpayload_plugin_init (plugin))
+    return FALSE;
+  if (!gst_rtpcndepayload_plugin_init (plugin))
+    return FALSE;
+#ifdef ENABLE_G729
+  if (!gst_rtp_g729_pay_plugin_init (plugin))    
+    return FALSE;
+  if (!gst_rtp_g729_depay_plugin_init (plugin))    
+    return FALSE;
+#endif
 
  return TRUE;
 }
